<?php

namespace Drupal\video_toolbox\Form;

use Drupal\Core\Url;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Drupal\video_toolbox\Traits\VideoFormDependencyInjectionTrait;

/**
 * Provides a Video toolbox form.
 */
class VideosUrlForm extends FormBase {

  use VideoFormDependencyInjectionTrait;

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'video_default_url';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $video_id = NULL) {

    $videogetter = $this->videoHandler;
    $videos = $videogetter->getVideoInfo($video_id);
    if ($videos == []) {
      return new RedirectResponse('/');
    }

    $uid = $this->account->id();
    $roles = $this->account->getRoles();

    /** @var \Drupal\file\Entity\File $file */
    $file = $this->fileStorage->load($videos[0]['fid']);
    $videourl = $file->createFileUrl();
    $form['video' . $video_id] = [
      '#type' => 'html_tag',
      '#tag' => 'video',
      '#attributes' => [
        'src' => $videourl,
        'allow' => 'fullscreen',
        'width' => 420,
        'height' => 315,
        'loading' => 'eager',
        'controls' => 'true',
      ],
    ];

    if ($videos[0]['uid'] != $uid || !array_search("administrator", $roles)) {
      return $form;
    }

    $form['description'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Description'),
      '#required' => TRUE,
      '#default_value' => $videos[0]['description'],
    ];

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save'),
      '#validate' => ['::validateFormSave'],
    ];

    $form['download'] = [
      '#type' => 'submit',
      '#value' => $this->t('Download'),
      '#submit' => ['::downloadSubmit'],
      '#validate' => ['::downloadValidate'],
    ];
    $form['delete'] = [
      '#type' => 'submit',
      '#value' => $this->t('Delete'),
      '#attributes' => ['onclick' => 'if(!confirm("Delete video?")){return false;}'],
      '#submit' => ['::delete'],
    ];

    $form['fid'] = [
      '#type' => 'hidden',
      '#value' => $videos[0]['fid'],
    ];

    return $form;
  }

  /**
   * Validate Form inputs only for the Save button.
   */
  public function validateFormSave(array &$form, FormStateInterface $form_state) {
    $desc = $form_state->getValue('description');
    if (is_string($desc) && mb_strlen($desc) < 10) {
      $form_state->setErrorByName('description', $this->t('Description should be at least 10 characters.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {

    $desc = $form_state->getValue('description');
    $fid = $form_state->getValue('fid');
    $videogetter = $this->videoHandler;
    $videogetter->updateVideos($desc, $fid);

    $this->messenger()->addStatus($this->t('Description saved'));
  }

  /**
   * Validate Download submission.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   */
  public function downloadValidate(array &$form, FormStateInterface $form_state) {
    // Used just to override default validation for the time being.
  }

  /**
   * Submit Handler for download button.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   */
  public function downloadSubmit(array &$form, FormStateInterface $form_state) {
    $videogetter = $this->videoHandler;
    // Get value from form and use downloadVideo method.
    $fid = $form_state->getValue('fid');
    $response = $videogetter->downloadVideo($fid);
    // Will set the response of the form to the response returned.
    $form_state->setResponse($response);
  }

  /**
   * Delete the video.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   */
  public function delete(array &$form, FormStateInterface $form_state) {
    $videoService = $this->videoHandler;
    $fid = $form_state->getValue('fid');
    $videoService->deleteVideo($fid);
    // Redirect to reports.
    $url = Url::fromRoute('video_toolbox.report');
    $url = $url->toString();
    $response = new RedirectResponse($url);
    $response->send();

  }

}
